@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Hunspell FLAG management
 
 # ***** BEGIN LICENSE BLOCK *****
@@ -39,6 +38,20 @@
 
 
 _flags = [
+    'onlyincompound',
+    # 체언
+    'substantive',
+    'substantive_t',                       # 받침으로 끝나는 체언
+    'substantive_v',                       # 모음으로 끝나는 체언
+    'substantive_v_not_pronoun_exception', # 모음으로 끝나는 체언 대명사 나/너/저 제외
+    # 대명사
+    'pronoun',
+    'pronoun_plural',                      # '-들'붙을 수 있는 대명사
+    'pronoun_exception',                   # 대명사 나/너/저
+    # 명사
+    'noun',
+    'noun_t',              # 받침 있는 명사
+    'noun_v',              # 받침 없는 명사
     # 아라비아숫자
     'digit',
     # 단위 명사, 숫자와 붙여 쓸 수 있음
@@ -63,15 +76,23 @@ _flags = [
     'auxiliary_eul',
     # 한자어 숫자, 자리수별
     'number_1', 'number_10', 'number_100', 'number_1000', 'number_10000',
+    # 우리말 숫자, 자리수별
+    'knumber_1', 'knumber_10',
     # 용언 활용
     ('endings', 500),
     # 조사 모음
     ('josas', 100),
     # 서술 조사
     'josa_ida',
-    'josa_ida_t', # 받침으로 시작
+    'josa_ida_t',               # 받침으로 시작
+    'josa_ida_noun',            # 서술격조사명사형 (이었음)
+    'josa_ida_noun_v',          # 서술격조사명사형, 모음 뒤 (였음)
+    'ida_eo',
+    'ida_eun',
+    'ida_eul',
 ]
 
+
 def _define_flags():
     count = 1
     for flag in _flags:
