@@ -125,26 +125,26 @@ class JosaClass:
         JosaClass.next_flag += 1
 
     def match(self, word, pos, props):
-        pos_base = pos.split(':')[0]
-        if pos_base == '특수':
-            pos_base = ':'.join(pos.split(':')[0:2])
-
         if self.notafter:
-            if (word, '#' + pos_base) in self.notafter:
+            if (word, '#' + pos) in self.notafter:
                 return False
         if self.after:
-            # 부사는 상태부사, 성상부사, 정도부사, 양태부사만 보조사 허용
-            if pos_base == '부사':
-                if pos in ['부사:상태', '부사:성상', '부사:정도', '부사:양태']:
-                    pass
-                else:
-                    return False
+            if pos == '부사':
+                # FIXME - # 부사는 상태부사, 성상부사, 정도부사, 양태부사만 보조사 허용해야 하는데..
+                # if pos_detail in ['부사:상태', '부사:성상', '부사:정도', '부사:양태']:
+                #     pass
+                # else:
+                #     return False
+                pass
 
-            if ('#' + pos_base) in self.after:
+            if pos in ('명사', '대명사', '수사', '특수:숫자',
+                       '특수:알파벳', '특수:복수접미사'):
+                if '#체언' in self.after:
+                    return True
+
+            if ('#' + pos) in self.after:
                 return True
-            elif ('#' + pos) in self.after:
-                return True
-            elif (word, '#' + pos_base) in self.after:
+            elif (word, '#' + pos) in self.after:
                 return True
             else:
                 return False
@@ -176,21 +176,12 @@ groups = {}
 # '이' 주격/보격 조사
 groups['이'] = [
     JosaClass(rules=[('이', COND_T_ALL, '')],
-              after=['#명사', '#대명사',
-                     '#특수:숫자', '#특수:알파벳',
-                     '#특수:수:1', '#특수:수:10', '#특수:수:100', '#특수:수:1000',
-                     '#특수:고유수:1', '#특수:고유수:10',
-                     ]
+              after=['#체언'],
     ),
-    # 대명사 '-거'+'이' -> '게'
+    # 대명사 '-것'+'이' -> '게'
     JosaClass(
-        rules=[(V_E, ENC('거'), V_EO)],
-        after=[('거', '#대명사'),
-               ('그거', '#대명사'),
-               ('요거', '#대명사'),
-               ('이거', '#대명사'),
-               ('저거', '#대명사'),
-               ],
+        rules=[(V_E, ENC('것'), V_EO + T_SIOS)],
+        after=['#대명사'],
     ),
 ]
 
@@ -198,11 +189,7 @@ groups['이'] = [
 groups['가'] = [
     JosaClass(
         rules=[('가', COND_V_ALL, '')],
-        after=['#명사', '#대명사',
-               '#특수:숫자', '#특수:알파벳',
-               '#특수:수:1', '#특수:수:10', '#특수:수:100', '#특수:수:1000',
-               '#특수:고유수:1', '#특수:고유수:10',
-               ],
+        after=['#체언'],
         notafter=[('나', '#대명사'),
                   ('너', '#대명사'),
                   ('저', '#대명사'),
@@ -229,7 +216,6 @@ groups['!대명사+의'] = [
     # 대명사 '나'+'의' -> '내', '너'+'의' -> '네', '저'+'의' -> '제'
     JosaClass(
         rules=[(V_AE, V_A, V_A),
-               (V_E, V_EO, V_EO),
                (V_E, V_EO, V_EO),
                ],
         after=[('나', '#대명사'),
@@ -239,6 +225,30 @@ groups['!대명사+의'] = [
     ),
 ]
 
+# 대명사 '나'+'에게' -> '내게', '너'+'에게' -> '네게', '저'+'에게' -> '제게'
+
+_rules=[(V_AE+'게', V_A, V_A),
+        (V_E+'게', V_EO, V_EO)]
+_suffix_list = ['까지', '는', T_NIEUN,  '나',  '도',  '만',  '서',  '서는',  '서도',  '서만']
+_rules_suffix = []
+for _suffix in _suffix_list:
+    _rules_suffix += [(r[0]+_suffix,r[1],r[2]) for r in _rules]
+_rules += _rules_suffix
+
+groups['!대명사+에게'] = [
+    # 대명사 '나'+'에게' -> '내게', '너'+'에게' -> '네게', '저'+'에게' -> '제게'
+    JosaClass(
+        rules=_rules,
+        after=[('나', '#대명사'),
+               ('너', '#대명사'),
+               ('저', '#대명사'),
+               ],
+    ),
+]
+
+# 보조사 붙임
+
+
 # '-ㄴ', '-ㄹ' 형태로 줄여진 구어체
 groups['!종성줄임'] = [
     JosaClass(
@@ -282,12 +292,7 @@ groups['!보조사'] = [
                ('은', COND_T_ALL, ''), ('는', COND_V_ALL, ''),
                ('은커녕', COND_T_ALL, ''), ('는커녕', COND_V_ALL, ''),
                ],
-        after=['#부사',
-               '#명사', '#대명사', '#수사',
-               '#특수:숫자', '#특수:알파벳',
-               '#특수:수:1', '#특수:수:10', '#특수:수:100', '#특수:수:1000',
-               '#특수:고유수:1', '#특수:고유수:10'
-               ],
+        after=['#부사', '#체언'],
     ),
 ]
 
@@ -423,11 +428,7 @@ groups['*'] = [
          ('한테도', COND_ALL, ''),
          ('한테서', COND_ALL, ''),
          ],
-        after=['#명사', '#대명사', '#수사',
-               '#특수:숫자', '#특수:알파벳',
-               '#특수:수:1', '#특수:수:10', '#특수:수:100', '#특수:수:1000',
-               '#특수:고유수:1', '#특수:고유수:10',
-               ],
+        after=['#체언'],
     ),
 ]
 
@@ -443,15 +444,11 @@ def find_flags(word, pos, props):
     for klass in klasses:
         if klass.match(word, pos, props):
             result.append(klass.flag)
-    if pos.startswith('명사'):
+
+    if pos in ('명사', '대명사', '수사', '특수:숫자',
+               '특수:알파벳', '특수:복수접미사'):
         result.append(josa_ida_flag)
-    elif pos in ['대명사', '특수:복수접미사', '특수:알파벳', '특수:숫자']:
-        result.append(josa_ida_flag)
-    elif pos.startswith('특수:수:'):
-        result.append(josa_ida_flag)
-    elif pos.startswith('특수:고유수:'):
-        result.append(josa_ida_flag)
-    if (pos == '대명사'):
+    if pos == '대명사':
         result.append(josa_ida_t_flag)
     return result
 
@@ -487,12 +484,15 @@ def get_ida_rules(flagaliases):
             elif NFD(c)[:2] == NFD('이'):
                 ida_josas_t.append((NFD(c)[2:], COND_V_ALL))
 
+    result = []
     if len(ida_josas) > 0:
-        result = ['SFX %d Y %d' % (josa_ida_flag, len(ida_josas))]
+        result.append('# 서술격 조사')
+        result.append('SFX %d Y %d' % (josa_ida_flag, len(ida_josas)))
         for (sfx, cond) in ida_josas:
             result.append('SFX %d 0 %s %s' % (josa_ida_flag, ENC(sfx), cond))
 
     if len(ida_josas_t) > 0:
+        result.append('# 서술격 조사 받침 뒤')
         result.append('SFX %d Y %d' % (josa_ida_t_flag, len(ida_josas_t)))
         for (sfx, cond) in ida_josas_t:
             result.append('SFX %d 0 %s %s' % (josa_ida_t_flag, ENC(sfx), cond))
