@@ -37,6 +37,7 @@
 # ***** END LICENSE BLOCK *****
 
 import sys
+import copy
 import unicodedata
 import json
 
@@ -44,6 +45,7 @@ import config
 import suffix
 import josa
 import encoding
+import jamo
 from flags import *
 
 
@@ -62,12 +64,19 @@ def progress(s):
     return sys.stderr.write('Progress: ' + s + '...\n')
 
 
+def status(s):
+    return sys.stderr.write('Status: ' + s + '...\n')
+
+
 class Word:
     def __init__(self):
         self.word = ''
         self.pos = ''
+        self.pos_detail = ''
         self.props = []
-        self.stem = ''
+        self.stem = None
+        self.pronounce = None
+
         self.flags = []
         self.flags_alias = -1
         self.morph_alias = -1
@@ -75,59 +84,108 @@ class Word:
     def __hash__(self):
         return (self.word + self.pos).__hash__()
 
+    # set element
+    def __eq__(self, other):
+        if self.word != other.word:
+            return False
+        elif self.pos != other.pos:
+            return False
+        elif sorted(self.props) !=  sorted(other.props):
+            return False
+        else:
+            return True
+
     # to make it orderable
     def __lt__(self, other):
         if self.word < other.word:
             return True
         if self.pos < other.pos:
             return True
-        # FIXME: 이렇게 하면 순서가 다를텐데. set에서 뭐가 먼저 나올지 알고...
-        for prop in other.props:
-            if prop not in self.props:
-                return True
+        if sorted(self.props) < sorted(other.props):
+            return True
         return False
 
     def __repr__(self):
         return 'Word %s pos:%s' % (self.word, self.pos)
 
-    def attach_flags(word):
+    def ends_with_vowel(self):
+        if self.pronounce:
+            lastsyl = self.pronounce[-1]
+        else:
+            lastsyl = self.word[-1]
+        return ENC(lastsyl)[-1] in jamo.V_ALL
+
+    def ends_with_consonant(self):
+        return not self.ends_with_vowel()
+
+    def attach_flags(self):
         pos_default_flags = {
-            '명사': [],
-            '대명사': [],
-            '특수:복수접미사': [plural_suffix_flag],
+            '명사': [substantive_flag, noun_flag],
+            '대명사': [substantive_flag, pronoun_flag],
+            '수사': [substantive_flag],
+            '특수:복수접미사': [substantive_flag, plural_suffix_flag,
+                                onlyincompound_flag],
             '특수:알파벳': [alpha_flag],
             '특수:숫자': [digit_flag],
-            '특수:수:1': [number_1_flag],
-            '특수:수:10': [number_10_flag],
-            '특수:수:100': [number_100_flag],
-            '특수:수:1000': [number_1000_flag],
-            '특수:수:10000': [number_10000_flag],
-            '특수:고유수:1': [knumber_1_flag],
-            '특수:고유수:10': [knumber_10_flag],
             '특수:금지어': [forbidden_flag],
-            '내부:활용:-어': [conjugation_eo_flag],
-            '내부:활용:-은': [conjugation_eun_flag],
-            '내부:활용:-을': [conjugation_eul_flag],
+            '내부:활용:-어': [conjugation_eo_flag, onlyincompound_flag],
+            '내부:활용:-은': [conjugation_eun_flag, onlyincompound_flag],
+            '내부:활용:-을': [conjugation_eul_flag, onlyincompound_flag],
+            '내부:이다:-어': [ida_eo_flag, onlyincompound_flag],
+            '내부:이다:-은': [ida_eun_flag, onlyincompound_flag],
+            '내부:이다:-을': [ida_eul_flag, onlyincompound_flag],
         }
         try:
-            word.flags = pos_default_flags[word.pos]
+            self.flags = pos_default_flags[self.pos]
         except KeyError:
             pass
-        if word.pos == '동사' or word.pos == '형용사':
-            word.flags += suffix.find_flags(word.word, word.pos, word.props)
-        word.flags += josa.find_flags(word.word, word.pos, word.props)
+
+        pos_detail_default_flags = {
+            '명사:의존:단위성': [counter_flag],
+        }
+        try:
+            self.flags += pos_detail_default_flags[self.pos_detail]
+        except KeyError:
+            pass
+
+        if self.pos in ('명사', '대명사', '수사'):
+            if self.ends_with_vowel():
+                self.flags += [substantive_v_flag]
+                if self.pos == '명사':
+                    self.flags += [noun_v_flag]
+            else:
+                self.flags += [substantive_t_flag]
+                if self.pos == '명사':
+                    self.flags += [noun_t_flag]
+            if self.pos == '대명사':
+                if self.word == '너' or self.word == '나':
+                    pass
+                else:
+                    self.flags += [pronoun_plural_flag]
+
+        elif self.pos == '동사' or self.pos == '형용사':
+            self.flags += suffix.find_flags(self.word, self.pos, self.props)
+        self.flags += josa.find_flags(self.word, self.pos, self.props)
+
         prop_default_flags = {
             '단위명사': [counter_flag],
             '보조용언:-어': [auxiliary_eo_flag],
             '보조용언:-은': [auxiliary_eun_flag],
             '보조용언:-을': [auxiliary_eul_flag],
+            '수:1': [number_1_flag],
+            '수:10': [number_10_flag],
+            '수:100': [number_100_flag],
+            '수:1000': [number_1000_flag],
+            '수:10000': [number_10000_flag],
+            '고유수:1': [knumber_1_flag],
+            '고유수:10': [knumber_10_flag],
         }
-        for prop in word.props:
+        for prop in self.props:
             try:
-                word.flags += prop_default_flags[prop]
+                self.flags += prop_default_flags[prop]
             except KeyError:
                 pass
-        word.flags.sort()
+        self.flags.sort()
 
 
 class Dictionary:
@@ -151,16 +209,26 @@ class Dictionary:
         for entry in d['entries']:
             w = Word()
             w.word = entry['word']
-            w.pos = entry['pos']
+            w.pos_detail = entry['pos']
+
+            base = w.pos_detail.split(':')[0]
+            if base == '특수':
+                w.pos = w.pos_detail
+            else:
+                w.pos = base
+
             if 'props' in entry:
                 w.props = entry['props']
             if 'stem' in entry:
                 w.stem = entry['stem']
-            dic.add(w)
+            if 'pronounce' in entry:
+                w.pronounce = entry['pronounce']
+            self.add(w)
 
     def process(self):
-        progress('복수형 확장')
-        self.expand_plurals()
+        progress('중복 제거')
+        self.remove_duplicates()
+        status('중복 제거 후 단어 수: %d' % len(self.words))
         if config.expand_auxiliary_attached:
             progress('플래그 계산')
             self.attach_flags()
@@ -184,7 +252,8 @@ class Dictionary:
 
     def output_dic(self, outfile):
         outfile.write('%d\n' % len(self.words))
-        for word in sorted(list(self.words)):
+        lines = []
+        for word in self.words:
             line = '%s' % word.word
             if word.flags_alias > 0:
                 line += ('/%d' % word.flags_alias)
@@ -194,8 +263,9 @@ class Dictionary:
             if config.output_word_morph:
                 if word.morph_alias > 0:
                     line += (' %d' % word.morph_alias)
-            line += '\n'
-            outfile.write(ENC(line))
+            lines.append(ENC(line))
+        lines.sort()
+        outfile.write('\n'.join(lines))
 
     def output_aff(self, outfile):
         from string import Template
@@ -218,24 +288,41 @@ class Dictionary:
              'trychars': aff.TRYCHARS,
              'MAP': aff.MAP_DEFINES,
              'REP': aff.REP_DEFINES,
+             'onlyincompound_flag': str(onlyincompound_flag),
              'COMPOUNDRULE': aff.COMPOUNDRULE_DEFINES,
              'JOSA': josa_str,
              'SUFFIX': suffix_str, }
+
+        if config.required_hunspell_version < (1, 3, 1):
+            d['suggest_settings'] = ''
+        else:
+            l = ('MAXCPDSUGS 4',
+                 'MAXNGRAMSUGS 4',
+                 'MAXDIFF 0',
+                 'COMPOUNDMORESUFFIXES')
+            d['suggest_settings'] = '\n'.join(l)
+
         outfile.write(template.substitute(d))
 
     def get_AF(self):
         aliases = self.flag_aliases
-        result = 'AF %d\n' % len(aliases)
-        for flags in aliases:
-            result += 'AF %s\n' % ','.join('%d' % f for f in flags)
-        return result
+        if len(aliases) > 0:
+            result = 'AF %d\n' % len(aliases)
+            for flags in aliases:
+                result += 'AF %s\n' % ','.join('%d' % f for f in flags)
+            return result
+        else:
+            return ''
 
     def get_AM(self):
         aliases = self.morph_aliases
-        result = 'AM %d\n' % len(aliases)
-        for morph in aliases:
-            result += 'AM %s\n' % morph
-        return result
+        if len(aliases) > 0:
+            result = 'AM %d\n' % len(aliases)
+            for morph in aliases:
+                result += 'AM %s\n' % morph
+            return result
+        else:
+            return ''
 
     ######################################################################
 
@@ -246,6 +333,9 @@ class Dictionary:
             if word.flags:
                 if word.flags not in aliases:
                     aliases.append(word.flags)
+        aliases.sort()
+        for word in self.words:
+            if word.flags:
                 word.flags_alias = aliases.index(word.flags) + 1
         self.flag_aliases = aliases
 
@@ -259,25 +349,29 @@ class Dictionary:
                 word.morph_alias = aliases.index(morph) + 1
         self.morph_aliases = aliases
 
-    def expand_plurals(self):
-        new_words = []
+    def remove_duplicates(self):
+        remove_list = []
         for word in self.words:
-            pos_base = word.pos.split(':')[0]
-            if pos_base == '명사':
-                if word.pos != '명사:보통:비서술성':
-                    continue
-            elif word.pos == '대명사':
-                if word.word == '너' or word.word == '나':
-                    continue
-                elif '가산명사' not in word.props:
-                    continue
-            new_word = Word()
-            new_word.word = word.word + '들'
-            new_word.pos = word.pos
-            new_word.props = [p for p in word.props if p != '가산명사']
-            new_word.stem = word.word
-            new_words.append(new_word)
-        self.append(new_words)
+            if word.pos in ('동사', '형용사'):
+                if '보조용언:-어' in word.props:
+                    have_aux = True
+                    wc = copy.deepcopy(word)
+                    wc.props.remove('보조용언:-어')
+                elif '보조용언:-은' in word.props:
+                    have_aux = True
+                    wc = copy.deepcopy(word)
+                    wc.props.remove('보조용언:-은')
+                elif '보조용언:-을' in word.props:
+                    have_aux = True
+                    wc = copy.deepcopy(word)
+                    wc.props.remove('보조용언:-을')
+                else:
+                    have_aux = False
+                if have_aux and wc in self.words:
+                    remove_list.append(wc)
+
+        for word in remove_list:
+            self.words.remove(word)
 
     def expand_auxiliary(self):
         new_words = []
@@ -316,6 +410,50 @@ class Dictionary:
                         new_word.stem = verb.word
                         new_word.pos = '내부:활용:' + form
                         new_words.append(new_word)
+        # 서술격 조사
+        new_word = Word()
+        new_word.word = '이어'
+        new_word.pos = '내부:이다:-어'
+        new_word.stem = '이다'
+        new_word.props = []
+        new_words.append(new_word)
+        new_word = Word()
+        new_word.word = '이시어'
+        new_word.pos = '내부:이다:-어'
+        new_word.stem = '이다'
+        new_word.props = []
+        new_words.append(new_word)
+        new_word = Word()
+        new_word.word = '이셔'
+        new_word.pos = '내부:이다:-어'
+        new_word.stem = '이다'
+        new_word.props = []
+        new_words.append(new_word)
+        new_word = Word()
+        new_word.word = '인'
+        new_word.pos = '내부:이다:-은'
+        new_word.stem = '이다'
+        new_word.props = []
+        new_words.append(new_word)
+        new_word = Word()
+        new_word.word = '이신'
+        new_word.pos = '내부:이다:-은'
+        new_word.stem = '이다'
+        new_word.props = []
+        new_words.append(new_word)
+        new_word = Word()
+        new_word.word = '일'
+        new_word.pos = '내부:이다:-을'
+        new_word.stem = '이다'
+        new_word.props = []
+        new_words.append(new_word)
+        new_word = Word()
+        new_word.word = '이실'
+        new_word.pos = '내부:이다:-을'
+        new_word.stem = '이다'
+        new_word.props = []
+        new_words.append(new_word)
+
         self.append(new_words)
 
 if __name__ == '__main__':
